/*****************************************************************************
 *
 *	spi.c - CC3000 Host Driver Implementation.
 *	Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 *
 *	Redistribution and use in source and binary forms, with or without
 *	modification, are permitted provided that the following conditions
 *	are met:
 *
 *		Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *		Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in the
 *		documentation and/or other materials provided with the
 *		distribution.
 *
 *		Neither the name of Texas Instruments Incorporated nor the names of
 *		its contributors may be used to endorse or promote products derived
 *		from this software without specific prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *	OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

//*****************************************************************************
//
//! \addtogroup link_buff_api
//! @{
//
//*****************************************************************************
#include "hci.h"
#include "spi.h"

#define FWT_DELAY							 4000

#define SPI_WINDOW_SIZE				 1024
#define READ										3
#define WRITE									 1

#define HI(value)							 (((value) & 0xFF00) >> 8)
#define LO(value)							 ((value) & 0x00FF)

#define ASSERT_CS()						(Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0x1, 8, false))

#define DEASSERT_CS()					(Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0x1, 8, true))

#define HEADERS_SIZE_EVNT			(SPI_HEADER_SIZE + 5)

#define SPI_HEADER_SIZE			(5)

#define WLAN_IRQ_PORT				0
#define WLAN_IRQ_PIN				14
#define	WLAN_IRQ_PIN_INT4		4

#define WLAN_ENABLE_PORT			0
#define WLAN_ENABLE_PIN				7

#define 	eSPI_STATE_POWERUP 				 (0)
#define 	eSPI_STATE_INITIALIZED			 (1)
#define 	eSPI_STATE_IDLE					 (2)
#define 	eSPI_STATE_WRITE_IRQ		 		 (3)
#define 	eSPI_STATE_WRITE_FIRST_PORTION	 (4)
#define 	eSPI_STATE_WRITE_EOT			 (5)
#define 	eSPI_STATE_READ_IRQ				 (6)
#define 	eSPI_STATE_READ_FIRST_PORTION	 (7)
#define 	eSPI_STATE_READ_EOT				 (8)

// The magic number that resides at the end of the TX/RX buffer (1 byte after 
// the allocated size) for the purpose of detection of the overrun. The location
// of the memory where the magic number	resides shall never be written. 
// In case it is written - the overrun occurred and either receive function or
// send function will stuck forever.

#define CC3000_BUFFER_MAGIC_NUMBER (0xDE)

///////////////////////////////////////////////////////////////////////////////////////////////////////////
//#pragma is used for determine the memory location for a specific variable.														///				
//__no_init is used to prevent the buffer initialization in order to prevent hardware WDT expiration		///
// before entering to 'main()'.																																				 ///
//for every IDE, different syntax exists :					1.	 __CCS__ for CCS v5														 ///
//																									2.	__IAR_SYSTEMS_ICC__ for IAR Embedded Workbench	///
// *CCS does not initialize variables - therefore, __no_init is not needed.														 ///
///////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct {
	gcSpiHandleRx SPIRxHandler;

	unsigned short usTxPacketLength;
	unsigned short usRxPacketLength;
	unsigned long ulSpiState;
	unsigned char *pTxPacket;
	unsigned char *pRxPacket;

} tSpiInformation;

tSpiInformation sSpiInformation;

uint8_t wlan_rx_buffer[CC3000_RX_BUFFER_SIZE];
uint8_t wlan_tx_buffer[CC3000_TX_BUFFER_SIZE];
uint8_t chBuffer[CC3000_RX_BUFFER_SIZE];

static Chip_SSP_DATA_SETUP_T xf_setup;
static volatile uint8_t isXferCompleted = 0;

// Static buffer for 5 bytes of SPI HEADER
unsigned char tSpiReadHeader[] = { READ, 0, 0, 0, 0 };

void SpiWriteDataSynchronous(const unsigned char *data, unsigned short size);
void SpiWriteAsync(const unsigned char *data, unsigned short size);
void SpiReadData(unsigned char *data, unsigned short size);
void SpiDisableInterrupts(void);
void SpiResumeSpi(void);

//*****************************************************************************
//	
//!	SpiSysDelay
//! 
//!	@param	ulTicks ticks
//! 
//!	@return none
//! 
//!	@brief	This function delay for a few sys ticks.
//!					
// 
//*****************************************************************************

void SpiSysDelay(uint32_t ulTicks) {
	while (ulTicks--) {
	}
}

//*****************************************************************************
//
//!	SpiClose
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	Close the SPI interface
//
//*****************************************************************************
void SpiClose(void) {
	if (sSpiInformation.pRxPacket) {
		sSpiInformation.pRxPacket = 0;
	}

	//	Disable Interrupt in GPIOA module...
	tSLInformation.WlanInterruptDisable();

	// Disable interrupt for SPI IRQ and SSI module
	Chip_SSP_Disable(LPC_SSP1 );
	NVIC_DisableIRQ(SSP1_IRQn);
	NVIC_DisableIRQ(PIN_INT4_IRQn);
}

//*****************************************************************************
//
//!	SpiOpen
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	Open the SPI interface
//
//*****************************************************************************
void SpiOpen(gcSpiHandleRx pfRxHandler) {
	sSpiInformation.ulSpiState = eSPI_STATE_POWERUP;

	sSpiInformation.SPIRxHandler = pfRxHandler;
	sSpiInformation.usTxPacketLength = 0;
	sSpiInformation.pRxPacket = NULL;
	sSpiInformation.pRxPacket = wlan_rx_buffer;
	sSpiInformation.usRxPacketLength = 0;
	wlan_rx_buffer[CC3000_RX_BUFFER_SIZE - 1] = CC3000_BUFFER_MAGIC_NUMBER;
	wlan_tx_buffer[CC3000_TX_BUFFER_SIZE - 1] = CC3000_BUFFER_MAGIC_NUMBER;

	// Enable interrupt on the GPIOA ping of WLAN IRQ
	tSLInformation.WlanInterruptEnable();

	// Enable interrupts in NVIC
	Chip_SSP_Enable(LPC_SSP1 );
	//NVIC_EnableIRQ(SSP1_IRQn);
	NVIC_EnableIRQ(PIN_INT4_IRQn);
}

//*****************************************************************************
//
//!	init_spi
//!
//!	@param	buffer
//!
//!	@return none
//!
//!	@brief	initializes an SPI interface
//
//*****************************************************************************

void GPIOIntInit() {
	Chip_SCU_PinMuxSet(0x2, 10,
			(SCU_MODE_PULLDOWN | SCU_MODE_INBUFF_EN | SCU_MODE_ZIF_DIS
					| SCU_MODE_FUNC0)); /* P2_10->GPIO0[14]*/

	Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, WLAN_IRQ_PORT, WLAN_IRQ_PIN);
	Chip_SCU_GPIOIntPinSel(WLAN_IRQ_PIN_INT4, WLAN_IRQ_PORT, WLAN_IRQ_PIN);

	Chip_PININT_SetPinModeEdge(LPC_GPIO_PIN_INT, PININTCH4);
	Chip_PININT_EnableIntLow(LPC_GPIO_PIN_INT, PININTCH4);
	Chip_PININT_DisableIntHigh(LPC_GPIO_PIN_INT, PININTCH4);

	NVIC_EnableIRQ(PIN_INT4_IRQn); /* enable GPIO interrupt 4 */

	/* Enable CC3000 */
	Chip_SCU_PinMuxSet(0x2, 7,
			(SCU_MODE_PULLDOWN | SCU_MODE_ZIF_DIS | SCU_MODE_FUNC0)); /* P2_7->GPIO0[7]*/

	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, WLAN_ENABLE_PORT, WLAN_ENABLE_PIN);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, WLAN_ENABLE_PORT, WLAN_ENABLE_PIN,
			false);

	Chip_SCU_PinMuxSet(0x1, 5, (SCU_PINIO_FAST | SCU_MODE_FUNC0)); /* P2_7->GPIO0[7]*/

	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 1, 8);

	DEASSERT_CS();
}

int SPIInit(void) {

	/* Initializing SPI */
	Board_SSP_Init(LPC_SSP1 );
	Chip_SSP_Init(LPC_SSP1 );

	Chip_SSP_Set_Mode(LPC_SSP1, SSP_MODE_MASTER);
	Chip_SSP_SetFormat(LPC_SSP1, SSP_BITS_8, SSP_FRAMEFORMAT_SPI,
			SSP_CLOCK_MODE0);

	Chip_SSP_Enable(LPC_SSP1 );

	/* Enable SSP interrupt */
	Chip_SSP_Int_Disable(LPC_SSP1 );
	NVIC_EnableIRQ(SSP1_IRQn);

	GPIOIntInit();
	DEBUGOUT("SPI Initialized");
	return (ESUCCESS);
}

//*****************************************************************************
//
//!	SpiFirstWrite
//!
//!	@param	ucBuf buffer to write
//!	@param	ucBuf buffer's length
//!
//!	@return none
//!
//!	@brief	Spi first write operation
//
//*****************************************************************************
long SpiFirstWrite(unsigned char *ucBuf, unsigned short usLength) {
	// workaround for first transaction
	DEBUGOUT("First write\r\n");
	Chip_SSP_Int_FlushData(LPC_SSP1 );
	ASSERT_CS();

	SpiSysDelay(FWT_DELAY / 4);

	// SPI writes first 4 bytes of data
	SpiWriteDataSynchronous(ucBuf, 4);
	while (Chip_SSP_GetStatus(LPC_SSP1, SSP_STAT_BSY))
		;
	SpiSysDelay(FWT_DELAY / 4);

	SpiWriteDataSynchronous(ucBuf + 4, usLength - 4);

	// From this point on - operate in a regular way

	while (Chip_SSP_GetStatus(LPC_SSP1, SSP_STAT_BSY))
		;
	sSpiInformation.ulSpiState = eSPI_STATE_IDLE;
	DEASSERT_CS();

	return (0);
}

//*****************************************************************************
//
//! SpiWrite
//!
//!	@param	pUserBuffer buffer to write
//!	@param	pUserBuffer buffer's length
//!
//!	@return none
//!
//!	@brief	Spi write operation
//
//*****************************************************************************
long SpiWrite(unsigned char *pUserBuffer, unsigned short usLength) {
	unsigned char ucPad = 0;

	// Figure out the total length of the packet in order to figure out if there is padding or not
	if (!(usLength & 0x0001)) {
		ucPad++;
	}

	pUserBuffer[0] = WRITE;
	pUserBuffer[1] = HI(usLength + ucPad);
	pUserBuffer[2] = LO(usLength + ucPad);
	pUserBuffer[3] = 0;
	pUserBuffer[4] = 0;

	usLength += (SPI_HEADER_SIZE + ucPad);

	// The magic number that resides at the end of the TX/RX buffer (1 byte after the allocated size)
	// for the purpose of detection of the overrun. If the magic number is overwritten - buffer overrun
	// occurred - and we will stuck here forever!
	if (wlan_tx_buffer[CC3000_TX_BUFFER_SIZE - 1] != CC3000_BUFFER_MAGIC_NUMBER) {
		while (1)
			;
	}

	if (sSpiInformation.ulSpiState == eSPI_STATE_POWERUP) {
		while (sSpiInformation.ulSpiState != eSPI_STATE_INITIALIZED)
			;
	}

	if (sSpiInformation.ulSpiState == eSPI_STATE_INITIALIZED) {

		// This is time for first TX/RX transactions over SPI: the IRQ is down - so need to send read buffer size command
		SpiFirstWrite(pUserBuffer, usLength);

		// Due to the fact that we are currently implementing a blocking situation
		// here we will wait till end of transaction
	} else {

		// We need to prevent here race that can occur in case 2 back to back packets are sent to the 
		// device, so the state will move to IDLE and once again to not IDLE due to IRQ

		while (sSpiInformation.ulSpiState != eSPI_STATE_IDLE) {
			;
		}

		sSpiInformation.ulSpiState = eSPI_STATE_WRITE_IRQ;
		sSpiInformation.pRxPacket = pUserBuffer;
		sSpiInformation.usTxPacketLength = usLength;

		// Assert the CS line and wait till SSI IRQ line is active and then initialize write operation		 
		Chip_SSP_Int_FlushData(LPC_SSP1 );
		ASSERT_CS();
		SpiSysDelay(FWT_DELAY / 4);
	}

	// Due to the fact that we are currently implementing a blocking situation
	// here we will wait till end of transaction
	while (eSPI_STATE_IDLE != sSpiInformation.ulSpiState) {
		;
	}
	return (0);
}

//*****************************************************************************
//
//! This function enter point for write flow
//!
//!	@param	data	buffer to read to
//!	@param	size	buffer's size
//!
//!	@return none
//!
//!	@brief	Spi read operation
//
//*****************************************************************************
void SpiReadData(unsigned char *data, unsigned short size) {
	int i = 0;
	// The uDMA TX/RX channel must be disabled.
	//

	while (Chip_SSP_GetStatus(LPC_SSP1, SSP_STAT_BSY))
		;
	isXferCompleted = 0;
	xf_setup.rx_cnt = xf_setup.tx_cnt = 0;

	xf_setup.length = size;
	xf_setup.rx_data = data;
	xf_setup.tx_data = wlan_tx_buffer;

	DEBUGOUT("Reading %d bytes\r\n", size);

	//Chip_SSP_Int_Enable(LPC_SSP1 );	

	while (xf_setup.rx_cnt != xf_setup.length) {
		if (Chip_SSP_Int_RWFrames8Bits(LPC_SSP1, &xf_setup) == ERROR) {
			DEBUGOUT("Overrun while reading\r\n");
			SpiSysDelay(FWT_DELAY / 4);
		}
	}

	for (i = 0; i < size; i++) {
		DEBUGOUT("Read: %d\r\n", sSpiInformation.pRxPacket[i]);
	}
	//Chip_SSP_Int_Disable(LPC_SSP1 );	

}
//*****************************************************************************
//
//! SpiWriteAsync
//!
//!	@param	data	buffer to write
//!	@param	size	buffer's size
//!
//!	@return none
//!
//!	@brief	Spi write operation
//
//*****************************************************************************
void SpiWriteAsync(const unsigned char *data, unsigned short size) {

	// LMS uDMA is capable to transfer data in one shot only of size up to 1024
	// so in case of bigger transfer size will need to perform 2 transfers 
	// (scatter-gather uDMA	mode also can be used but it will not be very useful 
	// since CPU anyway is blocked on SPI write operation and can not do anything 
	// till the end.
	int i;
	Chip_SSP_Int_FlushData(LPC_SSP1 );
	ASSERT_CS();
	SpiSysDelay(FWT_DELAY / 4);

	isXferCompleted = 0;
	xf_setup.rx_cnt = xf_setup.tx_cnt = 0;

	xf_setup.length = size;
	xf_setup.tx_data = (unsigned char *) data;
	xf_setup.rx_data = wlan_rx_buffer;

	DEBUGOUT("Sending %d bytes\r\n", size);
	//for (i = 0; i < size; i++)
		//DEBUGOUT("Sending: %d\r\n", (char)xf_setup.tx_data[i]);

	//Chip_SSP_Int_Enable(LPC_SSP1 );	
	while (xf_setup.tx_cnt != xf_setup.length) {
		if (Chip_SSP_Int_RWFrames8Bits(LPC_SSP1, &xf_setup) == ERROR) {
			DEBUGOUT("Overrun while writing\r\n");
			SpiSysDelay(FWT_DELAY / 4);
		}
	}

	//Chip_SSP_Int_Disable(LPC_SSP1 );	
}

//*****************************************************************************
//
//! SpiWriteDataSynchronous
//!
//!	@param	data	buffer to write
//!	@param	size	buffer's size
//!
//!	@return none
//!
//!	@brief	Spi write operation
//
//*****************************************************************************
void SpiWriteDataSynchronous(const unsigned char *data, unsigned short size) {
	// we are in the SYNC write to SPI - we do not want to receive ANY SSI related
	// interrupts

	SpiWriteAsync(data, size);
}

//*****************************************************************************
//
//! SpiReadHeader
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	SPI read operation. first we read minimal 5 SPI header bytes and 5 
//!					 Event Data bytes
//
//*****************************************************************************
void SpiReadHeader(void) {
	//sSpiInformation.ulSpiState = eSPI_STATE_READ_IRQ;
	SpiReadData(sSpiInformation.pRxPacket, 10);
}

//*****************************************************************************
//
//! SpiReadDataCont
//!
//!	\param	None
//!
//!	\return None
//!
//!	\brief	This function processes received SPI Header and in accordance with 
//!					it - continues reading the packet
//
//*****************************************************************************
long SpiReadDataCont(void) {

	long data_to_recv;
	unsigned char *evnt_buff, type;

	DEBUGOUT("SpiReadDataCont\r\n");
	//determine what type of packet we have
	evnt_buff = sSpiInformation.pRxPacket;
	data_to_recv = 0;
	STREAM_TO_UINT8((char *)(evnt_buff + SPI_HEADER_SIZE),
			HCI_PACKET_TYPE_OFFSET, type);
	DEBUGOUT("Type:%d\r\n", type);

	switch (type) {
	case HCI_TYPE_DATA: {
		STREAM_TO_UINT16((char *)(evnt_buff + SPI_HEADER_SIZE),
				HCI_DATA_LENGTH_OFFSET, data_to_recv);

		// We need to read the rest of data..
		if (!((HEADERS_SIZE_EVNT + data_to_recv) & 1)) {
			data_to_recv++;
		}

		if (data_to_recv) {
			SpiReadData(evnt_buff + 10, data_to_recv);
		}

		//sSpiInformation.ulSpiState = eSPI_STATE_READ_EOT;

		break;
	}
	case HCI_TYPE_EVNT: {

		// Calculate the rest length of the data
		STREAM_TO_UINT8((char *)(evnt_buff + SPI_HEADER_SIZE),
				HCI_EVENT_LENGTH_OFFSET, data_to_recv);
		data_to_recv -= 1;

		// Add padding byte if needed
		if ((HEADERS_SIZE_EVNT + data_to_recv) & 1) {

			data_to_recv++;
		}

		if (data_to_recv) {
			SpiReadData(evnt_buff + 10, data_to_recv);
		}

		sSpiInformation.ulSpiState = eSPI_STATE_READ_EOT;
		break;
	}
	}

	return (data_to_recv);
}

//*****************************************************************************
//
//! SpiDisableInterrupts
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	Disable SPI interrupts
//
//*****************************************************************************

void SpiDisableInterrupts(void) {
	//Chip_PININT_DisableIntHigh(LPC_GPIO_PIN_INT, PININTCH4);
	//NVIC_DisableIRQ(PIN_INT4_IRQn);
}

//*****************************************************************************
//
//! SpiResumeSpi
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	Spi resume operation
//
//*****************************************************************************

void SpiResumeSpi(void) {
	//Chip_PININT_EnableIntHigh(LPC_GPIO_PIN_INT, PININTCH4);
	//NVIC_EnableIRQ(PIN_INT4_IRQn);
}

//*****************************************************************************
//
//! SpiTriggerRxProcessing
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	Processing operation for SPI RX
//
//*****************************************************************************
void SpiTriggerRxProcessing(void) {

	// Trigger Rx processing
	SpiDisableInterrupts();
	while (Chip_SSP_GetStatus(LPC_SSP1, SSP_STAT_BSY))
		;
	SpiSysDelay(FWT_DELAY / 4);
	DEASSERT_CS();

	// The magic number that resides at the end of the TX/RX buffer (1 byte after the allocated size)
	// for the purpose of detection of the overrun. If the magic number is overwritten - buffer overrun
	// occurred - and we will stuck here forever!
	if (sSpiInformation.pRxPacket[CC3000_RX_BUFFER_SIZE - 1]
			!= CC3000_BUFFER_MAGIC_NUMBER) {
		while (1)
			;
	}

	sSpiInformation.ulSpiState = eSPI_STATE_IDLE;
	sSpiInformation.SPIRxHandler(sSpiInformation.pRxPacket + SPI_HEADER_SIZE);
}

//*****************************************************************************
//
//! SSIContReadOperationw
//!
//!	@param	none
//!
//!	@return none
//!
//!	@brief	Spi read operation
//
//*****************************************************************************

void SSIContReadOperation(void) {

	// The header was read - continue with	the payload read
	if (!SpiReadDataCont()) {
		// All the data was read - finalize handling by switching to the task
		//	and calling from task Event Handler
		SpiTriggerRxProcessing();
	}
}

//*****************************************************************************
// 
//!	IntSpiGPIOHandler
//! 
//!	@param	none
//! 
//!	@return none
//! 
//!	@brief	GPIO A interrupt handler. When the external SSI WLAN device is
//!					ready to interact with Host CPU it generates an interrupt signal.
//!					After that Host CPU has registered this interrupt request
//!					it set the corresponding /CS in active state.
// 
//*****************************************************************************
void GPIO4_IRQHandler(void) {
	DEBUGOUT("GPIO4_IRQHandler\r\n");
	DEBUGOUT("%d\r\n", Chip_PININT_GetIntStatus(LPC_GPIO_PIN_INT ));
	DEBUGOUT("%d\r\n", PININTCH4);

	if (Chip_PININT_GetIntStatus(LPC_GPIO_PIN_INT ) & PININTCH4) {
		Chip_PININT_ClearIntStatus(LPC_GPIO_PIN_INT, PININTCH4);

		if (sSpiInformation.ulSpiState == eSPI_STATE_POWERUP) {
			// This means IRQ line was low call a callback of HCI Layer to 
			// inform on event
			sSpiInformation.ulSpiState = eSPI_STATE_INITIALIZED;
		} else if (sSpiInformation.ulSpiState == eSPI_STATE_IDLE) {
			sSpiInformation.ulSpiState = eSPI_STATE_READ_IRQ;

			// IRQ line goes down - we are start reception 
			Chip_SSP_Int_FlushData(LPC_SSP1 );
			ASSERT_CS();
			SpiSysDelay(FWT_DELAY / 4);
			// Wait for TX/RX Compete which will come as DMA interrupt
			SpiReadHeader();
			sSpiInformation.ulSpiState = eSPI_STATE_READ_EOT;

			SSIContReadOperation();
		} else if (sSpiInformation.ulSpiState == eSPI_STATE_WRITE_IRQ) {
			SpiWriteAsync(sSpiInformation.pRxPacket,
					sSpiInformation.usTxPacketLength);

			sSpiInformation.ulSpiState = eSPI_STATE_IDLE;
			while (Chip_SSP_GetStatus(LPC_SSP1, SSP_STAT_BSY))
				;
			SpiSysDelay(FWT_DELAY / 4);
			DEASSERT_CS();
		}
	}
}

//*****************************************************************************
//
//! SpiIntHandler
//!
//! @param	none
//!
//! @return none
//!
//! @brief	SSI interrupt handler. Get command/data packet from the external
//!				 WLAN device and pass it through the UART to the PC
//
//*****************************************************************************
void SSP1_IRQHandler(void) {
	DEBUGOUT("SSP0_IRQHandler\r\n");
	DEBUGOUT("VALUE: %u \r\n", Chip_SSP_GetIntStatus(LPC_SSP1));

	if (Chip_SSP_GetIntStatus(LPC_SSP1 )) {
		//Chip_SSP_Int_Disable(LPC_SSP1 );
		if ((xf_setup.rx_cnt != xf_setup.length)
				|| (xf_setup.tx_cnt != xf_setup.length)) {
			if (Chip_SSP_Int_RWFrames8Bits(LPC_SSP1, &xf_setup) == ERROR)
				DEBUGOUT("Overrun while reading/write\r\n");
		}

		if ((xf_setup.rx_cnt != xf_setup.length)
				|| (xf_setup.tx_cnt != xf_setup.length)) {
			//Chip_SSP_Int_Enable(LPC_SSP1);	/* enable all interrupts */
		} else {
			isXferCompleted = 1;
		}
	} else
		Chip_SSP_Int_Disable(LPC_SSP1 );
}

long ReadWlanInterruptPin(void) {
	// Return the status of IRQ, P2.4 
	return Chip_GPIO_GetPinState(LPC_GPIO_PORT, WLAN_IRQ_PORT, WLAN_IRQ_PIN);
}

//*****************************************************************************
//
//! WlanInterruptEnable
//!
//! @param	none
//!
//! @return none
//!
//! @brief	Enable waln IrQ pin
//
//*****************************************************************************

void WlanInterruptEnable() {
	NVIC_EnableIRQ(PIN_INT4_IRQn);
}

//*****************************************************************************
//
//! WlanInterruptDisable
//!
//! @param	none
//!
//! @return none
//!
//! @brief	Disable waln IrQ pin
//
//*****************************************************************************

void WlanInterruptDisable() {
	NVIC_DisableIRQ(PIN_INT4_IRQn);
}

//*****************************************************************************
//
//! WriteWlanPin
//!
//! @param	val value to write to wlan pin
//!
//! @return none
//!
//! @brief	write value to wlan pin
//
//*****************************************************************************

void WriteWlanPin(uint8_t trueFalse) {
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, WLAN_ENABLE_PORT, WLAN_ENABLE_PIN,
			trueFalse);								 //	high
}

//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
